<?php
/**
 * app/plugins/calendar/models/events.php
 *
 * This class is used for the manipulation of the calendars database table.
 *
 * @package TP Calendar
 * @version 3.0
 * @author  Joey Kimsey <Joey@thetempusproject.com>
 * @link    https://TheTempusProject.com
 * @license https://opensource.org/licenses/MIT [MIT LICENSE]
 */
namespace TheTempusProject\Models;

use TheTempusProject\Bedrock\Classes\Config;
use TheTempusProject\Bedrock\Functions\Check;
use TheTempusProject\Bedrock\Functions\Date;
use TheTempusProject\Canary\Bin\Canary as Debug;
use TheTempusProject\Classes\DatabaseModel;
use TheTempusProject\TheTempusProject as App;
use TheTempusProject\Houdini\Classes\Filters;
use TheTempusProject\Bedrock\Classes\CustomException;

class Calendars extends DatabaseModel {
    public $tableName = 'calendars';
    public $databaseMatrix = [
        [ 'title', 'varchar', '256' ],
        [ 'color', 'varchar', '48' ],
        [ 'privacy', 'varchar', '48' ],
        [ 'description', 'text', '' ],
        [ 'createdBy', 'int', '11' ],
        [ 'createdAt', 'int', '11' ],
        [ 'timezone', 'varchar', '256' ],
    ];

    /**
     * The model constructor.
     */
    public function __construct() {
        parent::__construct();
    }

    public function create( $title, $description = '', $timezone = '', $color = 'default' ) {
        if ( ! Check::dataTitle( $title ) ) {
            Debug::info( 'Calendars: illegal title.' );
            return false;
        }
        if ( empty( $timezone ) ) {
            $timezone = Date::getTimezone();
        }
        $fields = [
            'title' => $title,
            'description' => $description,
            'timezone' => $timezone,
            'color' => $color,
            'createdBy' => App::$activeUser->ID,
            'createdAt' => time(),
        ];
        if ( ! self::$db->insert( $this->tableName, $fields ) ) {
            new CustomException( 'calendarCreate' );
            Debug::error( "Calendar: not created " . var_export($fields,true) );
            return false;
        }
        return self::$db->lastId();
    }

    public function update( $id, $title, $description = '', $timezone = '', $color = 'default' ) {
        if ( empty( $timezone ) ) {
            $timezone = Date::getTimezone();
        }
        if ( !Check::id( $id ) ) {
            Debug::info( 'Calendars: illegal ID.' );
            return false;
        }
        if ( !Check::dataTitle( $title ) ) {
            Debug::info( 'Calendars: illegal title.' );
            return false;
        }
        $fields = [
            'title' => $title,
            'description' => $description,
            'timezone' => $timezone,
            'color' => $color,
        ];
        if ( !self::$db->update( $this->tableName, $id, $fields ) ) {
            new CustomException( 'calendarUpdate' );
            Debug::error( "Calendar: $id not updated: $fields" );
            return false;
        }
        return true;
    }

    public function byUser( $limit = null ) {
        $whereClause = ['createdBy', '=', App::$activeUser->ID];
        if ( empty( $limit ) ) {
            $calendars = self::$db->get( $this->tableName, $whereClause );
        } else {
            $calendars = self::$db->get( $this->tableName, $whereClause, 'ID', 'DESC', [0, $limit] );
        }
        if ( !$calendars->count() ) {
            Debug::info( 'No Calendars found.' );
            return false;
        }
        return $this->filter( $calendars->results() );
    }

    public function getName( $id ) {
        $calendar = self::findById( $id );
        if (false == $calendar) {
            return 'unknown';
        }
        return $calendar->title;
    }

    public function getColor( $id ) {
        $calendar = self::findById( $id );
        if (false == $calendar) {
            return 'default';
        }
        return $calendar->color;
    }

    public function simpleByUser() {
        $whereClause = ['createdBy', '=', App::$activeUser->ID];
        $calendars = self::$db->get( $this->tableName, $whereClause );
        if ( !$calendars->count() ) {
            Debug::warn( 'Could not find any calendars' );
            return false;
        }
        
        $calendars = $calendars->results();
        $out = [];
        foreach ( $calendars as &$calendar ) {
            $out[ $calendar->title ] = $calendar->ID;
        }
        return $out;
    }

    public function simpleObjectByUser() {
        $whereClause = ['createdBy', '=', App::$activeUser->ID];
        $calendars = self::$db->get( $this->tableName, $whereClause );
        if ( !$calendars->count() ) {
            Debug::warn( 'Could not find any calendars' );
            return false;
        }
        
        $calendars = $calendars->results();
        $out = [];
        foreach ( $calendars as &$calendar ) {
            $obj = new \stdClass();
            $obj->title = $calendar->title;
            $obj->ID = $calendar->ID;
            $out[] = $obj;
        }
        return $out;
    }
}
